@@ -5,6 +5,7 @@ use crate::data::input::{
     PointerButton, RawInput, TouchDeviceId, ViewportInfo,
 };
 use crate::{
+    SafeAreaInsets,
     emath::{NumExt as _, Pos2, Rect, Vec2, vec2},
     util::History,
 };
@@ -17,6 +18,37 @@ pub use crate::Key;
 pub use touch_state::MultiTouchInfo;
 use touch_state::TouchState;
 
+#[derive(Clone, Copy, Debug, Default, PartialEq, Eq)]
+#[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
+pub enum SurrenderFocusOn {
+    /// Surrender focus if the user _presses_ somewhere outside the focused widget.
+    Presses,
+
+    /// Surrender focus if the user _clicks_ somewhere outside the focused widget.
+    #[default]
+    Clicks,
+
+    /// Never surrender focus.
+    Never,
+}
+
+impl SurrenderFocusOn {
+    pub fn ui(&mut self, ui: &mut crate::Ui) {
+        ui.horizontal(|ui| {
+            ui.selectable_value(self, Self::Presses, "Presses")
+                .on_hover_text(
+                    "Surrender focus if the user presses somewhere outside the focused widget.",
+                );
+            ui.selectable_value(self, Self::Clicks, "Clicks")
+                .on_hover_text(
+                    "Surrender focus if the user clicks somewhere outside the focused widget.",
+                );
+            ui.selectable_value(self, Self::Never, "Never")
+                .on_hover_text("Never surrender focus.");
+        });
+    }
+}
+
 /// Options for input state handling.
 #[derive(Clone, Copy, Debug, PartialEq)]
 #[cfg_attr(feature = "serde", derive(serde::Deserialize, serde::Serialize))]
@@ -58,6 +90,9 @@ pub struct InputOptions {
     /// and when combined with [`Self::zoom_modifier`] it will result in zooming
     /// on only the vertical axis.
     pub vertical_scroll_modifier: Modifiers,
+
+    /// When should we surrender focus from the focused widget?
+    pub surrender_focus_on: SurrenderFocusOn,
 }
 
 impl Default for InputOptions {
@@ -79,6 +114,7 @@ impl Default for InputOptions {
             zoom_modifier: Modifiers::COMMAND,
             horizontal_scroll_modifier: Modifiers::SHIFT,
             vertical_scroll_modifier: Modifiers::ALT,
+            surrender_focus_on: SurrenderFocusOn::default(),
         }
     }
 }
@@ -95,6 +131,7 @@ impl InputOptions {
             zoom_modifier,
             horizontal_scroll_modifier,
             vertical_scroll_modifier,
+            surrender_focus_on,
         } = self;
         crate::Grid::new("InputOptions")
             .num_columns(2)
@@ -155,6 +192,10 @@ impl InputOptions {
                 vertical_scroll_modifier.ui(ui);
                 ui.end_row();
 
+                ui.label("surrender_focus_on");
+                surrender_focus_on.ui(ui);
+                ui.end_row();
+
             });
     }
 }
@@ -227,9 +268,17 @@ pub struct InputState {
     /// * `zoom > 1`: pinch spread
     zoom_factor_delta: f32,
 
+    /// Rotation in radians this frame, measuring clockwise (e.g. from a rotation gesture).
+    rotation_radians: f32,
+
     // ----------------------------------------------
     /// Position and size of the egui area.
-    pub screen_rect: Rect,
+    ///
+    /// This is including the area that may be covered by the `safe_area_insets`.
+    viewport_rect: Rect,
+
+    /// The safe area insets, subtracted from the `viewport_rect` in [`Self::content_rect`].
+    safe_area_insets: SafeAreaInsets,
 
     /// Also known as device pixel ratio, > 1 for high resolution screens.
     pub pixels_per_point: f32,
@@ -314,8 +363,10 @@ impl Default for InputState {
             raw_scroll_delta: Vec2::ZERO,
             smooth_scroll_delta: Vec2::ZERO,
             zoom_factor_delta: 1.0,
+            rotation_radians: 0.0,
 
-            screen_rect: Rect::from_min_size(Default::default(), vec2(10_000.0, 10_000.0)),
+            viewport_rect: Rect::from_min_size(Default::default(), vec2(10_000.0, 10_000.0)),
+            safe_area_insets: Default::default(),
             pixels_per_point: 1.0,
             max_texture_side: 2048,
             time: 0.0,
@@ -353,7 +404,8 @@ impl InputState {
             new.predicted_dt
         };
 
-        let screen_rect = new.screen_rect.unwrap_or(self.screen_rect);
+        let safe_area_insets = new.safe_area_insets.unwrap_or(self.safe_area_insets);
+        let viewport_rect = new.screen_rect.unwrap_or(self.viewport_rect);
         self.create_touch_states_for_new_devices(&new.events);
         for touch_state in self.touch_states.values_mut() {
             touch_state.begin_pass(time, &new, self.pointer.interact_pos);
@@ -362,6 +414,7 @@ impl InputState {
 
         let mut keys_down = self.keys_down;
         let mut zoom_factor_delta = 1.0; // TODO(emilk): smoothing for zoom factor
+        let mut rotation_radians = 0.0;
         let mut raw_scroll_delta = Vec2::ZERO;
 
         let mut unprocessed_scroll_delta = self.unprocessed_scroll_delta;
@@ -392,7 +445,7 @@ impl InputState {
                     let mut delta = match unit {
                         MouseWheelUnit::Point => *delta,
                         MouseWheelUnit::Line => options.line_scroll_speed * *delta,
-                        MouseWheelUnit::Page => screen_rect.height() * *delta,
+                        MouseWheelUnit::Page => viewport_rect.height() * *delta,
                     };
 
                     let is_horizontal = modifiers.matches_any(options.horizontal_scroll_modifier);
@@ -440,6 +493,9 @@ impl InputState {
                 Event::Zoom(factor) => {
                     zoom_factor_delta *= *factor;
                 }
+                Event::Rotate(radians) => {
+                    rotation_radians += *radians;
+                }
                 Event::WindowFocused(false) => {
                     // Example: pressing `Cmd+S` brings up a save-dialog (e.g. using rfd),
                     // but we get no key-up event for the `S` key (in winit).
@@ -502,8 +558,10 @@ impl InputState {
             raw_scroll_delta,
             smooth_scroll_delta,
             zoom_factor_delta,
+            rotation_radians,
 
-            screen_rect,
+            viewport_rect,
+            safe_area_insets,
             pixels_per_point,
             max_texture_side: new.max_texture_side.unwrap_or(self.max_texture_side),
             time,
@@ -525,9 +583,48 @@ impl InputState {
         self.raw.viewport()
     }
 
+    /// Returns the region of the screen that is safe for content rendering
+    ///
+    /// Returns the `viewport_rect` with the `safe_area_insets` removed.
+    ///
+    /// If you want to render behind e.g. the dynamic island on iOS, use [`Self::viewport_rect`].
+    ///
+    /// See also [`RawInput::safe_area_insets`].
     #[inline(always)]
+    pub fn content_rect(&self) -> Rect {
+        self.viewport_rect - self.safe_area_insets
+    }
+
+    /// Returns the full area available to egui, including parts that might be partially covered,
+    /// for example, by the OS status bar or notches (see [`Self::safe_area_insets`]).
+    ///
+    /// Usually you want to use [`Self::content_rect`] instead.
+    ///
+    /// This rectangle includes e.g. the dynamic island on iOS.
+    /// If you want to only render _below_ the that (not behind), then you should use
+    /// [`Self::content_rect`] instead.
+    ///
+    /// See also [`RawInput::safe_area_insets`].
+    pub fn viewport_rect(&self) -> Rect {
+        self.viewport_rect
+    }
+
+    /// Position and size of the egui area.
+    #[deprecated(
+        note = "screen_rect has been split into viewport_rect() and content_rect(). You likely should use content_rect()"
+    )]
     pub fn screen_rect(&self) -> Rect {
-        self.screen_rect
+        self.content_rect()
+    }
+
+    /// Get the safe area insets.
+    ///
+    /// This represents the area of the screen covered by status bars, navigation controls, notches,
+    /// or other items that obscure part of the screen.
+    ///
+    /// See [`Self::content_rect`] to get the `viewport_rect` with the safe area insets removed.
+    pub fn safe_area_insets(&self) -> SafeAreaInsets {
+        self.safe_area_insets
     }
 
     /// Uniform zoom scale factor this frame (e.g. from ctrl-scroll or pinch gesture).
@@ -591,6 +688,26 @@ impl InputState {
         }
     }
 
+    /// Rotation in radians this frame, measuring clockwise (e.g. from a rotation gesture).
+    #[inline(always)]
+    pub fn rotation_delta(&self) -> f32 {
+        self.multi_touch()
+            .map_or(self.rotation_radians, |touch| touch.rotation_delta)
+    }
+
+    /// Panning translation in pixels this frame (e.g. from scrolling or a pan gesture)
+    ///
+    /// The delta indicates how the **content** should move.
+    ///
+    /// A positive X-value indicates the content is being moved right, as when swiping right on a touch-screen or track-pad with natural scrolling.
+    ///
+    /// A positive Y-value indicates the content is being moved down, as when swiping down on a touch-screen or track-pad with natural scrolling.
+    #[inline(always)]
+    pub fn translation_delta(&self) -> Vec2 {
+        self.multi_touch()
+            .map_or(self.smooth_scroll_delta, |touch| touch.translation_delta)
+    }
+
     /// How long has it been (in seconds) since the use last scrolled?
     #[inline(always)]
     pub fn time_since_last_scroll(&self) -> f32 {
@@ -815,10 +932,11 @@ impl InputState {
     ) -> impl Iterator<Item = &accesskit::ActionRequest> {
         let accesskit_id = id.accesskit_id();
         self.events.iter().filter_map(move |event| {
-            if let Event::AccessKitActionRequest(request) = event {
-                if request.target == accesskit_id && request.action == action {
-                    return Some(request);
-                }
+            if let Event::AccessKitActionRequest(request) = event
+                && request.target == accesskit_id
+                && request.action == action
+            {
+                return Some(request);
             }
             None
         })
@@ -832,10 +950,10 @@ impl InputState {
     ) {
         let accesskit_id = id.accesskit_id();
         self.events.retain(|event| {
-            if let Event::AccessKitActionRequest(request) = event {
-                if request.target == accesskit_id {
-                    return !consume(request);
-                }
+            if let Event::AccessKitActionRequest(request) = event
+                && request.target == accesskit_id
+            {
+                return !consume(request);
             }
             true
         });
@@ -1218,6 +1336,11 @@ impl PointerState {
         self.press_origin
     }
 
+    /// How far has the pointer moved since the start of the drag (if any)?
+    pub fn total_drag_delta(&self) -> Option<Vec2> {
+        Some(self.latest_pos? - self.press_origin?)
+    }
+
     /// When did the current click/drag originate?
     /// `None` if no mouse button is down.
     #[inline(always)]
@@ -1399,10 +1522,10 @@ impl PointerState {
                 return false;
             }
 
-            if let Some(press_start_time) = self.press_start_time {
-                if self.time - press_start_time > self.options.max_click_duration {
-                    return false;
-                }
+            if let Some(press_start_time) = self.press_start_time
+                && self.time - press_start_time > self.options.max_click_duration
+            {
+                return false;
             }
 
             true
@@ -1486,9 +1609,11 @@ impl InputState {
             unprocessed_scroll_delta_for_zoom,
             raw_scroll_delta,
             smooth_scroll_delta,
+            rotation_radians,
 
             zoom_factor_delta,
-            screen_rect,
+            viewport_rect,
+            safe_area_insets,
             pixels_per_point,
             max_texture_side,
             time,
@@ -1539,8 +1664,10 @@ impl InputState {
             "smooth_scroll_delta: {smooth_scroll_delta:?} points"
         ));
         ui.label(format!("zoom_factor_delta: {zoom_factor_delta:4.2}x"));
+        ui.label(format!("rotation_radians: {rotation_radians:.3} radians"));
 
-        ui.label(format!("screen_rect: {screen_rect:?} points"));
+        ui.label(format!("viewport_rect: {viewport_rect:?} points"));
+        ui.label(format!("safe_area_insets: {safe_area_insets:?} points"));
         ui.label(format!(
             "{pixels_per_point} physical pixels for each logical point"
         ));
