@@ -24,11 +24,7 @@ impl ShaderVersion {
         let shading_lang_string =
             unsafe { gl.get_parameter_string(glow::SHADING_LANGUAGE_VERSION) };
         let shader_version = Self::parse(&shading_lang_string);
-        log::debug!(
-            "Shader version: {:?} ({:?}).",
-            shader_version,
-            shading_lang_string
-        );
+        log::debug!("Shader version: {shader_version:?} ({shading_lang_string:?}).");
         shader_version
     }
 
