@@ -95,7 +95,7 @@ impl BoxElem {
     }
 
     /// Name of this box element.
-    #[allow(clippy::needless_pass_by_value)]
+    #[expect(clippy::needless_pass_by_value)]
     #[inline]
     pub fn name(mut self, name: impl ToString) -> Self {
         self.name = name.to_string();
