@@ -1,5 +1,5 @@
 //! Contains items that can be added to a plot.
-#![allow(clippy::type_complexity)] // TODO(emilk): simplify some of the callback types with type aliases
+#![expect(clippy::type_complexity)] // TODO(emilk): simplify some of the callback types with type aliases
 
 use std::{ops::RangeInclusive, sync::Arc};
 
@@ -54,7 +54,11 @@ macro_rules! builder_methods_for_base {
         /// Name of this plot item.
         ///
         /// This name will show up in the plot legend, if legends are turned on.
-        #[allow(clippy::needless_pass_by_value)]
+        ///
+        /// Setting the name via this method does not change the item's id, so you can use it to
+        /// change the name dynamically between frames without losing the item's state. You should
+        /// make sure the name passed to [`Self::new`] is unique and stable for each item, or
+        /// set unique and stable ids explicitly via [`Self::id`].
         #[inline]
         pub fn name(mut self, name: impl ToString) -> Self {
             self.base_mut().name = name.to_string();
@@ -79,7 +83,8 @@ macro_rules! builder_methods_for_base {
 
         /// Sets the id of this plot item.
         ///
-        /// By default the id is determined from the name, but it can be explicitly set to a different value.
+        /// By default the id is determined from the name passed to [`Self::new`], but it can be
+        /// explicitly set to a different value.
         #[inline]
         pub fn id(mut self, id: impl Into<Id>) -> Self {
             self.base_mut().id = id.into();
@@ -532,32 +537,39 @@ impl PlotItem for Line<'_> {
             let y = transform
                 .position_from_point(&PlotPoint::new(0.0, y_reference))
                 .y;
-            let mut fill_color = Rgba::from(stroke.color)
+            let default_fill_color = Rgba::from(stroke.color)
                 .to_opaque()
                 .multiply(fill_alpha)
                 .into();
+
+            let fill_color_for_point = |pos| {
+                if *gradient_fill && self.gradient_color.is_some() {
+                    Rgba::from(self
+                        .gradient_color
+                        .clone()
+                        .expect("Could not find gradient color callback")(
+                        transform.value_from_position(pos),
+                    ))
+                    .to_opaque()
+                    .multiply(fill_alpha)
+                    .into()
+                } else {
+                    default_fill_color
+                }
+            };
+
             let mut mesh = Mesh::default();
             let expected_intersections = 20;
             mesh.reserve_triangles((n_values - 1) * 2);
             mesh.reserve_vertices(n_values * 2 + expected_intersections);
             values_tf.windows(2).for_each(|w| {
-                if *gradient_fill && self.gradient_color.is_some() {
-                    fill_color = Rgba::from(self
-                        .gradient_color
-                        .clone()
-                        .expect("Could not find gradient color callback")(
-                        transform.value_from_position(w[1]),
-                    ))
-                    .to_opaque()
-                    .multiply(fill_alpha)
-                    .into();
-                }
+                let fill_color = fill_color_for_point(w[0]);
                 let i = mesh.vertices.len() as u32;
                 mesh.colored_vertex(w[0], fill_color);
                 mesh.colored_vertex(pos2(w[0].x, y), fill_color);
                 if let Some(x) = y_intersection(&w[0], &w[1], y) {
                     let point = pos2(x, y);
-                    mesh.colored_vertex(point, fill_color);
+                    mesh.colored_vertex(point, fill_color_for_point(point));
                     mesh.add_triangle(i, i + 1, i + 2);
                     mesh.add_triangle(i + 2, i + 3, i + 4);
                 } else {
@@ -566,6 +578,7 @@ impl PlotItem for Line<'_> {
                 }
             });
             let last = values_tf[n_values - 1];
+            let fill_color = fill_color_for_point(last);
             mesh.colored_vertex(last, fill_color);
             mesh.colored_vertex(pos2(last.x, y), fill_color);
             shapes.push(Shape::Mesh(std::sync::Arc::new(mesh)));
@@ -873,7 +886,6 @@ impl<'a> Points<'a> {
 }
 
 impl PlotItem for Points<'_> {
-    #[allow(clippy::too_many_lines)] // TODO(emilk): shorten this function
     fn shapes(&self, _ui: &Ui, transform: &PlotTransform, shapes: &mut Vec<Shape>) {
         let sqrt_3 = 3_f32.sqrt();
         let frac_sqrt_3_2 = 3_f32.sqrt() / 2.0;
@@ -1674,7 +1686,7 @@ fn add_rulers_and_text(
     let font_id = TextStyle::Body.resolve(plot.ui.style());
 
     let corner_value = elem.corner_value();
-    plot.ui.fonts(|f| {
+    plot.ui.fonts_mut(|f| {
         shapes.push(Shape::text(
             f,
             plot.transform.position_from_point(&corner_value) + vec2(3.0, -2.0),
@@ -1690,7 +1702,6 @@ fn add_rulers_and_text(
 /// and a label describing the coordinate.
 ///
 /// `value` is used to for text displaying X/Y coordinates.
-#[allow(clippy::too_many_arguments)]
 pub(super) fn rulers_and_tooltip_at_value(
     plot_area_response: &egui::Response,
     value: PlotPoint,
