@@ -173,6 +173,8 @@ pub struct Plot<'a> {
     height: Option<f32>,
     data_aspect: Option<f32>,
     view_aspect: Option<f32>,
+    invert_x: bool,
+    invert_y: bool,
 
     reset: bool,
 
@@ -221,6 +223,8 @@ impl<'a> Plot<'a> {
             height: None,
             data_aspect: None,
             view_aspect: None,
+            invert_x: false,
+            invert_y: false,
 
             reset: false,
 
@@ -273,6 +277,22 @@ impl<'a> Plot<'a> {
         self
     }
 
+    /// Set whether to invert the x-axis (i.e. positive values go to the left).
+    /// By default the x-axis is not inverted (i.e. positive values go to the right).
+    #[inline]
+    pub fn invert_x(mut self, invert: bool) -> Self {
+        self.invert_x = invert;
+        self
+    }
+
+    /// Set whether to invert the y-axis (i.e. positive values go down).
+    /// By default the y-axis is not inverted (i.e. positive values go up).
+    #[inline]
+    pub fn invert_y(mut self, invert: bool) -> Self {
+        self.invert_y = invert;
+        self
+    }
+
     /// Width of plot. By default a plot will fill the ui it is in.
     /// If you set [`Self::view_aspect`], the width can be calculated from the height.
     #[inline]
@@ -777,8 +797,7 @@ impl<'a> Plot<'a> {
         self.show_dyn(ui, Box::new(build_fn))
     }
 
-    #[allow(clippy::too_many_lines)] // TODO(emilk): shorten this function
-    #[allow(clippy::type_complexity)] // build_fn
+    #[expect(clippy::too_many_lines)] // TODO(emilk): shorten this function
     fn show_dyn<'b, R>(
         self,
         ui: &mut Ui,
@@ -803,6 +822,8 @@ impl<'a> Plot<'a> {
             mut min_size,
             data_aspect,
             view_aspect,
+            invert_x,
+            invert_y,
             mut show_x,
             mut show_y,
             label_formatter,
@@ -911,7 +932,7 @@ impl<'a> Plot<'a> {
                     let link_groups: &mut BoundsLinkGroups = data.get_temp_mut_or_default(Id::NULL);
                     link_groups.0.remove(name);
                 });
-            };
+            }
             None
         } else {
             PlotMemory::load(ui.ctx(), plot_id)
@@ -920,7 +941,12 @@ impl<'a> Plot<'a> {
             auto_bounds: default_auto_bounds,
             hovered_legend_item: None,
             hidden_items: Default::default(),
-            transform: PlotTransform::new(plot_rect, min_auto_bounds, center_axis),
+            transform: PlotTransform::new_with_invert_axis(
+                plot_rect,
+                min_auto_bounds,
+                center_axis,
+                Vec2b::new(invert_x, invert_y),
+            ),
             last_click_pos_for_zoom: None,
             x_axis_thickness: Default::default(),
             y_axis_thickness: Default::default(),
@@ -1024,9 +1050,9 @@ impl<'a> Plot<'a> {
                         bounds.set_y(&linked_bounds.bounds);
                         mem.auto_bounds.y = linked_bounds.auto_bounds.y;
                     }
-                };
+                }
             });
-        };
+        }
 
         // Allow double-clicking to reset to the initial bounds.
         if allow_double_click_reset && response.double_clicked() {
@@ -1094,7 +1120,12 @@ impl<'a> Plot<'a> {
             }
         }
 
-        mem.transform = PlotTransform::new(plot_rect, bounds, center_axis);
+        mem.transform = PlotTransform::new_with_invert_axis(
+            plot_rect,
+            bounds,
+            center_axis,
+            Vec2b::new(invert_x, invert_y),
+        );
 
         // Enforce aspect ratio
         if let Some(data_aspect) = data_aspect {
@@ -1713,7 +1744,6 @@ impl PreparedPlot<'_> {
     }
 
     fn paint_grid(&self, ui: &Ui, shapes: &mut Vec<(Shape, f32)>, axis: Axis, fade_range: Rangef) {
-        #![allow(clippy::collapsible_else_if)]
         let Self {
             transform,
             // axis_formatters,
@@ -1752,12 +1782,12 @@ impl PreparedPlot<'_> {
                     Axis::X => {
                         if !clamp_range.range_x().contains(&value_main) {
                             continue;
-                        };
+                        }
                     }
                     Axis::Y => {
                         if !clamp_range.range_y().contains(&value_main) {
                             continue;
-                        };
+                        }
                     }
                 }
             }
@@ -1946,14 +1976,15 @@ fn test_generate_marks() {
         gm(3.01, 0.01),
     ];
 
-    let mut problem = None;
-    if marks.len() != expected.len() {
-        problem = Some(format!(
+    let mut problem = if marks.len() == expected.len() {
+        None
+    } else {
+        Some(format!(
             "Different lengths: got {}, expected {}",
             marks.len(),
             expected.len()
-        ));
-    }
+        ))
+    };
 
     for (i, (a, b)) in marks.iter().zip(&expected).enumerate() {
         if !approx_eq(a, b) {
