@@ -91,8 +91,8 @@ impl PlotBounds {
     #[inline]
     pub fn center(&self) -> PlotPoint {
         [
-            (self.min[0] + self.max[0]) / 2.0,
-            (self.min[1] + self.max[1]) / 2.0,
+            emath::fast_midpoint(self.min[0], self.max[0]),
+            emath::fast_midpoint(self.min[1], self.max[1]),
         ]
         .into()
     }
@@ -276,6 +276,9 @@ pub struct PlotTransform {
 
     /// Whether to always center the x-range or y-range of the bounds.
     centered: Vec2b,
+
+    /// Whether to always invert the x and/or y axis
+    inverted_axis: Vec2b,
 }
 
 impl PlotTransform {
@@ -306,7 +309,7 @@ impl PlotTransform {
                     1.0
                 },
             );
-        };
+        }
 
         if !bounds.is_finite_y() {
             new_bounds.set_y(&PlotBounds::new_symmetrical(1.0));
@@ -319,15 +322,15 @@ impl PlotTransform {
                     1.0
                 },
             );
-        };
+        }
 
         // Scale axes so that the origin is in the center.
         if center_axis.x {
             new_bounds.make_x_symmetrical();
-        };
+        }
         if center_axis.y {
             new_bounds.make_y_symmetrical();
-        };
+        }
 
         debug_assert!(
             new_bounds.is_valid(),
@@ -338,9 +341,21 @@ impl PlotTransform {
             frame,
             bounds: new_bounds,
             centered: center_axis,
+            inverted_axis: Vec2b::new(false, false),
         }
     }
 
+    pub fn new_with_invert_axis(
+        frame: Rect,
+        bounds: PlotBounds,
+        center_axis: impl Into<Vec2b>,
+        invert_axis: impl Into<Vec2b>,
+    ) -> Self {
+        let mut new = Self::new(frame, bounds, center_axis);
+        new.inverted_axis = invert_axis.into();
+        new
+    }
+
     /// ui-space rectangle.
     #[inline]
     pub fn frame(&self) -> &Rect {
@@ -386,7 +401,11 @@ impl PlotTransform {
         remap(
             value,
             self.bounds.min[0]..=self.bounds.max[0],
-            (self.frame.left() as f64)..=(self.frame.right() as f64),
+            if self.inverted_axis[0] {
+                (self.frame.right() as f64)..=(self.frame.left() as f64)
+            } else {
+                (self.frame.left() as f64)..=(self.frame.right() as f64)
+            },
         ) as f32
     }
 
@@ -394,7 +413,12 @@ impl PlotTransform {
         remap(
             value,
             self.bounds.min[1]..=self.bounds.max[1],
-            (self.frame.bottom() as f64)..=(self.frame.top() as f64), // negated y axis!
+            // negated y axis by default
+            if self.inverted_axis[1] {
+                (self.frame.top() as f64)..=(self.frame.bottom() as f64)
+            } else {
+                (self.frame.bottom() as f64)..=(self.frame.top() as f64)
+            },
         ) as f32
     }
 
@@ -410,14 +434,24 @@ impl PlotTransform {
     pub fn value_from_position(&self, pos: Pos2) -> PlotPoint {
         let x = remap(
             pos.x as f64,
-            (self.frame.left() as f64)..=(self.frame.right() as f64),
+            if self.inverted_axis[0] {
+                (self.frame.right() as f64)..=(self.frame.left() as f64)
+            } else {
+                (self.frame.left() as f64)..=(self.frame.right() as f64)
+            },
             self.bounds.range_x(),
         );
         let y = remap(
             pos.y as f64,
-            (self.frame.bottom() as f64)..=(self.frame.top() as f64), // negated y axis!
+            // negated y axis by default
+            if self.inverted_axis[1] {
+                (self.frame.top() as f64)..=(self.frame.bottom() as f64)
+            } else {
+                (self.frame.bottom() as f64)..=(self.frame.top() as f64)
+            },
             self.bounds.range_y(),
         );
+
         PlotPoint::new(x, y)
     }
 
@@ -437,12 +471,14 @@ impl PlotTransform {
 
     /// delta position / delta value = how many ui points per step in the X axis in "plot space"
     pub fn dpos_dvalue_x(&self) -> f64 {
-        self.frame.width() as f64 / self.bounds.width()
+        let flip = if self.inverted_axis[0] { -1.0 } else { 1.0 };
+        flip * (self.frame.width() as f64) / self.bounds.width()
     }
 
     /// delta position / delta value = how many ui points per step in the Y axis in "plot space"
     pub fn dpos_dvalue_y(&self) -> f64 {
-        -self.frame.height() as f64 / self.bounds.height() // negated y axis!
+        let flip = if self.inverted_axis[1] { 1.0 } else { -1.0 };
+        flip * (self.frame.height() as f64) / self.bounds.height()
     }
 
     /// delta position / delta value = how many ui points per step in "plot space"
