@@ -1,4 +1,5 @@
-use fluent::{FluentAttribute, FluentMessage};
+use fluent::concurrent::FluentBundle;
+use fluent::{FluentAttribute, FluentMessage, FluentResource};
 use fluent_syntax::ast::{CallArguments, Expression, InlineExpression, Pattern, PatternElement};
 use i18n_embed::{fluent::FluentLanguageLoader, FileSystemAssets, LanguageLoader};
 use proc_macro::TokenStream;
@@ -9,6 +10,12 @@ use std::{
     path::Path,
     sync::OnceLock,
 };
+
+#[cfg(feature = "dashmap")]
+use dashmap::mapref::one::Ref;
+#[cfg(not(feature = "dashmap"))]
+use std::sync::{Arc, RwLock};
+
 use syn::{parse::Parse, parse_macro_input, spanned::Spanned};
 use unic_langid::LanguageIdentifier;
 
@@ -59,7 +66,7 @@ enum FlArgs {
     ///     arg3 = calc_value());
     /// ```
     KeyValuePairs {
-        specified_args: HashMap<syn::LitStr, Box<syn::Expr>>,
+        specified_args: Vec<(syn::LitStr, Box<syn::Expr>)>,
     },
     /// `fl!(LOADER, "message", "optional-attribute")` no arguments after the message id and optional attribute id.
     None,
@@ -76,7 +83,7 @@ impl Parse for FlArgs {
                 return Ok(FlArgs::HashMap(hash_map));
             }
 
-            let mut args_map: HashMap<syn::LitStr, Box<syn::Expr>> = HashMap::new();
+            let mut args: Vec<(syn::LitStr, Box<syn::Expr>)> = Vec::new();
 
             while let Ok(expr) = input.parse::<syn::ExprAssign>() {
                 let argument_name_ident_opt = match &*expr.left {
@@ -101,7 +108,10 @@ impl Parse for FlArgs {
 
                 let argument_value = expr.right;
 
-                if let Some(_duplicate) = args_map.insert(argument_name_lit_str, argument_value) {
+                if args
+                    .iter()
+                    .any(|(key, _value)| argument_name_lit_str == *key)
+                {
                     // There's no Clone implementation by default.
                     let argument_name_lit_str =
                         syn::LitStr::new(&argument_name_string, argument_name_ident.span());
@@ -113,20 +123,22 @@ impl Parse for FlArgs {
                         ),
                     ));
                 }
+                args.push((argument_name_lit_str, argument_value));
 
                 // parse the next comma if there is one
                 let _result = input.parse::<syn::Token![,]>();
             }
 
-            if args_map.is_empty() {
+            if args.is_empty() {
                 let span = match input.fork().parse::<syn::Expr>() {
                     Ok(expr) => expr.span(),
                     Err(_) => input.span(),
                 };
                 Err(syn::Error::new(span, "fl!() unable to parse args input"))
             } else {
+                args.sort_by_key(|(s, _)| s.value());
                 Ok(FlArgs::KeyValuePairs {
-                    specified_args: args_map,
+                    specified_args: args,
                 })
             }
         } else {
@@ -165,10 +177,57 @@ struct DomainSpecificData {
     _assets: FileSystemAssets,
 }
 
-fn domains() -> &'static dashmap::DashMap<String, DomainSpecificData> {
-    static DOMAINS: OnceLock<dashmap::DashMap<String, DomainSpecificData>> = OnceLock::new();
+#[derive(Default)]
+struct DomainsMap {
+    #[cfg(not(feature = "dashmap"))]
+    map: RwLock<HashMap<String, Arc<DomainSpecificData>>>,
 
-    DOMAINS.get_or_init(|| dashmap::DashMap::new())
+    #[cfg(feature = "dashmap")]
+    map: dashmap::DashMap<String, DomainSpecificData>,
+}
+
+#[cfg(feature = "dashmap")]
+impl DomainsMap {
+    fn get(&self, domain: &String) -> Option<Ref<String, DomainSpecificData>> {
+        self.map.get(domain)
+    }
+
+    fn entry_or_insert(
+        &self,
+        domain: &String,
+        data: DomainSpecificData,
+    ) -> Ref<String, DomainSpecificData> {
+        self.map.entry(domain.clone()).or_insert(data).downgrade()
+    }
+}
+
+#[cfg(not(feature = "dashmap"))]
+impl DomainsMap {
+    fn get(&self, domain: &String) -> Option<Arc<DomainSpecificData>> {
+        match self.map.read().unwrap().get(domain) {
+            None => None,
+            Some(data) => Some(data.clone()),
+        }
+    }
+
+    fn entry_or_insert(
+        &self,
+        domain: &String,
+        data: DomainSpecificData,
+    ) -> Arc<DomainSpecificData> {
+        self.map
+            .write()
+            .unwrap()
+            .entry(domain.clone())
+            .or_insert(Arc::new(data))
+            .clone()
+    }
+}
+
+fn domains() -> &'static DomainsMap {
+    static DOMAINS: OnceLock<DomainsMap> = OnceLock::new();
+
+    DOMAINS.get_or_init(|| DomainsMap::default())
 }
 
 /// A macro to obtain localized messages and optionally their attributes, and check the `message_id`, `attribute_id`
@@ -276,7 +335,7 @@ fn domains() -> &'static dashmap::DashMap<String, DomainSpecificData> {
 /// runtime using a [HashMap](std::collections::HashMap), using the
 /// same signature as in
 /// [FluentLanguageLoader::get_args()](i18n_embed::fluent::FluentLanguageLoader::get_args()).
-/// When using this method of specifying argments, they are not
+/// When using this method of specifying arguments, they are not
 /// checked at compile time.
 ///
 /// ### Example
@@ -421,7 +480,7 @@ pub fn fl(input: TokenStream) -> TokenStream {
             _assets: assets,
         };
 
-        domains().entry(domain.clone()).or_insert(data).downgrade()
+        domains().entry_or_insert(&domain, data)
     };
 
     let message_id_string = match &message_id {
@@ -504,8 +563,8 @@ pub fn fl(input: TokenStream) -> TokenStream {
                 if let Some(message_id_str) = &message_id_string {
                     checked_loader_has_message = domain_data
                         .loader
-                        .with_fluent_message(message_id_str, |message: FluentMessage<'_>| {
-                            check_message_args(message, &specified_args);
+                        .with_fluent_message_and_bundle(message_id_str, |message, bundle| {
+                            check_message_args(message, bundle, &specified_args);
                         })
                         .is_some();
                 }
@@ -524,11 +583,11 @@ pub fn fl(input: TokenStream) -> TokenStream {
             } else {
                 if let Some(message_id_str) = &message_id_string {
                     if let Some(attr_id_str) = &attr_str {
-                        let attr_res = domain_data.loader.with_fluent_message(
+                        let attr_res = domain_data.loader.with_fluent_message_and_bundle(
                             message_id_str,
-                            |message: FluentMessage<'_>| match message.get_attribute(attr_id_str) {
+                            |message, bundle| match message.get_attribute(attr_id_str) {
                                 Some(attr) => {
-                                    check_attribute_args(attr, &specified_args);
+                                    check_attribute_args(attr, bundle, &specified_args);
                                     true
                                 }
                                 None => false,
@@ -668,19 +727,22 @@ fn fuzzy_attribute_suggestions(
         .collect()
 }
 
-fn check_message_args(
+fn check_message_args<R>(
     message: FluentMessage<'_>,
-    specified_args: &HashMap<syn::LitStr, Box<syn::Expr>>,
-) {
+    bundle: &FluentBundle<R>,
+    specified_args: &Vec<(syn::LitStr, Box<syn::Expr>)>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     if let Some(pattern) = message.value() {
         let mut args = Vec::new();
-        args_from_pattern(pattern, &mut args);
+        args_from_pattern(pattern, bundle, &mut args);
 
         let args_set: HashSet<&str> = args.into_iter().collect();
 
         let key_args: Vec<String> = specified_args
-            .keys()
-            .map(|key| {
+            .iter()
+            .map(|(key, _value)| {
                 let arg = key.value();
 
                 if !args_set.contains(arg.as_str()) {
@@ -735,19 +797,22 @@ fn check_message_args(
     }
 }
 
-fn check_attribute_args(
+fn check_attribute_args<R>(
     attr: FluentAttribute<'_>,
-    specified_args: &HashMap<syn::LitStr, Box<syn::Expr>>,
-) {
+    bundle: &FluentBundle<R>,
+    specified_args: &Vec<(syn::LitStr, Box<syn::Expr>)>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     let pattern = attr.value();
     let mut args = Vec::new();
-    args_from_pattern(pattern, &mut args);
+    args_from_pattern(pattern, bundle, &mut args);
 
     let args_set: HashSet<&str> = args.into_iter().collect();
 
     let key_args: Vec<String> = specified_args
-        .keys()
-        .map(|key| {
+        .iter()
+        .map(|(key, _value)| {
             let arg = key.value();
 
             if !args_set.contains(arg.as_str()) {
@@ -801,56 +866,101 @@ fn check_attribute_args(
     }
 }
 
-fn args_from_pattern<S: Copy>(pattern: &Pattern<S>, args: &mut Vec<S>) {
+fn args_from_pattern<'m, R>(
+    pattern: &Pattern<&'m str>,
+    bundle: &'m FluentBundle<R>,
+    args: &mut Vec<&'m str>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     pattern.elements.iter().for_each(|element| {
         if let PatternElement::Placeable { expression } = element {
-            args_from_expression(expression, args)
+            args_from_expression(expression, bundle, args)
         }
     });
 }
 
-fn args_from_expression<S: Copy>(expr: &Expression<S>, args: &mut Vec<S>) {
+fn args_from_expression<'m, R>(
+    expr: &Expression<&'m str>,
+    bundle: &'m FluentBundle<R>,
+    args: &mut Vec<&'m str>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     match expr {
         Expression::Inline(inline_expr) => {
-            args_from_inline_expression(inline_expr, args);
+            args_from_inline_expression(inline_expr, bundle, args);
         }
         Expression::Select { selector, variants } => {
-            args_from_inline_expression(selector, args);
+            args_from_inline_expression(selector, bundle, args);
 
             variants.iter().for_each(|variant| {
-                args_from_pattern(&variant.value, args);
+                args_from_pattern(&variant.value, bundle, args);
             })
         }
     }
 }
 
-fn args_from_inline_expression<S: Copy>(inline_expr: &InlineExpression<S>, args: &mut Vec<S>) {
+fn args_from_inline_expression<'m, R>(
+    inline_expr: &InlineExpression<&'m str>,
+    bundle: &'m FluentBundle<R>,
+    args: &mut Vec<&'m str>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     match inline_expr {
         InlineExpression::FunctionReference {
             id: _,
             arguments: call_args,
         } => {
-            args_from_call_arguments(call_args, args);
+            args_from_call_arguments(call_args, bundle, args);
         }
         InlineExpression::TermReference {
             id: _,
             attribute: _,
             arguments: Some(call_args),
         } => {
-            args_from_call_arguments(call_args, args);
+            args_from_call_arguments(call_args, bundle, args);
         }
         InlineExpression::VariableReference { id } => args.push(id.name),
-        InlineExpression::Placeable { expression } => args_from_expression(expression, args),
+        InlineExpression::Placeable { expression } => {
+            args_from_expression(expression, bundle, args)
+        }
+        InlineExpression::MessageReference {
+            id,
+            attribute: None,
+        } => {
+            bundle
+                .get_message(&id.name)
+                .and_then(|m| m.value())
+                .map(|p| args_from_pattern(p, bundle, args));
+        }
+        InlineExpression::MessageReference {
+            id,
+            attribute: Some(attribute),
+        } => {
+            bundle
+                .get_message(&id.name)
+                .and_then(|m| m.get_attribute(&attribute.name))
+                .map(|m| m.value())
+                .map(|p| args_from_pattern(p, bundle, args));
+        }
         _ => {}
     }
 }
 
-fn args_from_call_arguments<S: Copy>(call_args: &CallArguments<S>, args: &mut Vec<S>) {
+fn args_from_call_arguments<'m, R>(
+    call_args: &CallArguments<&'m str>,
+    bundle: &'m FluentBundle<R>,
+    args: &mut Vec<&'m str>,
+) where
+    R: std::borrow::Borrow<FluentResource>,
+{
     call_args.positional.iter().for_each(|expr| {
-        args_from_inline_expression(expr, args);
+        args_from_inline_expression(expr, bundle, args);
     });
 
     call_args.named.iter().for_each(|named_arg| {
-        args_from_inline_expression(&named_arg.value, args);
+        args_from_inline_expression(&named_arg.value, bundle, args);
     })
 }
