@@ -3,9 +3,7 @@
 #[doc = include_str!("deref.md")]
 pub mod deref {}
 
-#[doc = include_str!("list.md")]
-#[doc = include_str!("list_data.md")]
-pub mod list {}
+pub mod list;
 
 #[doc = include_str!("CHANGELOG.md")]
 pub mod changelog {}
