@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: MIT
 
 #![cfg_attr(not(feature = "std"), no_std)]
+#![deny(unreachable_pub)]
 
 #[cfg(not(feature = "std"))]
 extern crate alloc;
@@ -15,6 +16,8 @@ mod sys;
 
 #[cfg(feature = "std")]
 pub use sys::{get_display_size, EventIter, Window};
+#[cfg(all(feature = "std", target_os = "redox"))]
+pub use sys::Surface;
 
 #[cfg(feature = "unifont")]
 pub static FONT: &[u8] = include_bytes!("../res/unifont.font");
@@ -42,6 +45,9 @@ pub enum WindowFlag {
     Unclosable,
 }
 
+#[derive(Clone, Copy, Debug)]
+pub enum SurfaceFlag {}
+
 #[derive(Clone, Copy, Debug)]
 pub enum Mode {
     Blend,     //Composite
