@@ -1,6 +1,7 @@
 // SPDX-License-Identifier: MIT
 
 use std::cell::{Cell, RefCell};
+use std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use std::sync::atomic::{AtomicUsize, Ordering};
 use std::{mem, ptr, slice};
 
@@ -81,6 +82,24 @@ impl Drop for Window {
     }
 }
 
+impl AsRawFd for Window {
+    fn as_raw_fd(&self) -> RawFd {
+        unimplemented!()
+    }
+}
+
+impl FromRawFd for Window {
+    unsafe fn from_raw_fd(_fd: RawFd) -> Window {
+        unimplemented!()
+    }
+}
+
+impl IntoRawFd for Window {
+    fn into_raw_fd(self) -> RawFd {
+        unimplemented!()
+    }
+}
+
 impl Renderer for Window {
     /// Get width
     fn width(&self) -> u32 {
@@ -419,13 +438,8 @@ impl Window {
     }
 
     fn get_mouse_position(&self) -> (i32, i32) {
-        unsafe {
-            let p_x: *mut i32 = libc::malloc(mem::size_of::<i32>()) as *mut i32;
-            let p_y: *mut i32 = libc::malloc(mem::size_of::<i32>()) as *mut i32;
-            sdl2_sys::SDL_GetMouseState(p_x, p_y);
-
-            (*p_x, *p_y)
-        }
+        let mouse_state = unsafe { &mut *EVENT_PUMP }.mouse_state();
+        (mouse_state.x(), mouse_state.y())
     }
 
     fn convert_event(&self, event: sdl2::event::Event) -> Vec<Event> {
@@ -451,9 +465,7 @@ impl Window {
                 events.push(Event::new());
             }
             sdl2::event::Event::Window { win_event, .. } => match win_event {
-                sdl2::event::WindowEvent::Moved(x, y) => {
-                    events.push(MoveEvent { x, y }.to_event())
-                }
+                sdl2::event::WindowEvent::Moved(x, y) => events.push(MoveEvent { x, y }.to_event()),
                 sdl2::event::WindowEvent::Resized(w, h) => events.push(
                     ResizeEvent {
                         width: w as u32,
@@ -495,12 +507,7 @@ impl Window {
             }
             sdl2::event::Event::TextInput { text, .. } => {
                 for character in text.chars() {
-                    events.push(
-                        TextInputEvent {
-                            character,
-                        }
-                        .to_event(),
-                    );
+                    events.push(TextInputEvent { character }.to_event());
                 }
             }
             sdl2::event::Event::KeyDown { scancode, .. } => {
